<?php
// $Id: customfilter.admin.inc,v 1.1.2.8.2.30 2009/07/27 06:26:26 kiam Exp $

/**
 * @file
 * Administration page callbacks for Custom filter.
 */
 
/**
 * @addtogroup customfilter
 * @{
 */

/***************************************************************************** 
 * Menu callbacks / form builders, submit/validate functions.
 ****************************************************************************/

/**
 * Return the export form.
 */
function customfilter_export_form(&$form_state) {
  $form = array();
  $filters = _customfilter_get_filters();
  $step = isset($form_state['storage']['step']) ? $form_state['storage']['step'] : 1;
  
  switch ($step) {
    case 1:
      if ($filters) {
        foreach ($filters as $filter) {
          $opt[$filter['fid']] = check_plain(t($filter['name']));
        }
        
        $form['filters'] = array(
          '#type' => 'radios',
          '#title' => t('Filters'),
          '#description' => 'Choose the filter to export.',
          '#options' => $opt,
        );
        
        $form['submit'] = array(
          '#type' => 'submit',
          '#value' => 'Export',
        );
      }
      else {
        $form['filters'] = array(
          '#value' => '<p>'. t('There are no custom filters defined.') .'</p>',
        );
      }
      break;
    
    case 2:
      $export_data = array();
      $fid = $form_state['values']['filters'];
      
      if ($filter = $filters[$fid]) {
        $export_data[] = '$filter = array(';
        $export_data[] = "  'fid' => _customfilter_map_filter_id('". $filter['fid'] ."'),";
        $export_data[] = "  'name' => base64_decode('". base64_encode($filter['name']) ."'),";
        $export_data[] = "  'cache' => {$filter['cache']},";
        $export_data[] = "  'description' => base64_decode('". base64_encode($filter['description']) ."'),";
        $export_data[] = "  'shorttips' => base64_decode('". base64_encode($filter['shorttips']) ."'),";
        $export_data[] = "  'longtips' => base64_decode('". base64_encode($filter['longtips']) ."'),";
        $export_data[] = ");";
        $export_data[] = '';
        
        $rules = array();
        _customfilter_rules_tree($rules, $fid, 0);
        foreach ($rules as $rule) {
          $export_data[] = '$rules[] = array(';
          $export_data[] = "  'rid' => _customfilter_map_rule_id('". $rule['rid'] ."'),";
          $export_data[] = "  'fid' => _customfilter_map_filter_id('". $filter['fid'] ."'),";
          $export_data[] = "  'prid' => _customfilter_map_rule_id('". $rule['prid'] ."'),";
          $export_data[] = "  'name' => base64_decode('". base64_encode($rule['name']) ."'),";
          $export_data[] = "  'description' => base64_decode('". base64_encode($rule['description']) ."'),";
          $export_data[] = "  'enabled' => {$rule['enabled']},";
          $export_data[] = "  'matches' => base64_decode('". base64_encode($rule['matches']) ."'),";
          $export_data[] = "  'pattern' => base64_decode('". base64_encode($rule['pattern']) ."'),";
          $export_data[] = "  'replacement' => base64_decode('". base64_encode($rule['replacement']) ."'),";
          $export_data[] = "  'code' => {$rule['code']},";
          $export_data[] = "  'weight' => {$rule['weight']},";
          $export_data[] = ");";
        }
      }
      
      if ($rows = count($export_data)) {
        $export_data = implode("\n", $export_data);
        
        $form['export'] = array(
          '#type' => 'textarea',
          '#title' => t('Export data'),
          '#description' => t('Copy the export text and paste it into the import form.'),
          '#cols' => 60,
          '#default_value' => $export_data,
          '#rows' => max(40, $rows),
        );
      }
      else {
        $form['export'] = array(
          '#value' => '<p>'. t('The selected filter has not been found in the database table.') .'</p>',
        );
      }
      break;
  }
  
  return $form;
}
 
function customfilter_export_form_submit($form, &$form_state) {
  $step = isset($form_state['storage']['step']) ? $form_state['storage']['step'] : 1;
  if ($step < 2) {
    $form_state['storage']['step'] = $step + 1;
  }
}

/**
 * Return the filter deleting form.
 */ 
function customfilter_filter_delete($form_state, $fid) {
  _customfilter_breadcrumb();
  
  $filters = _customfilter_get_filters();
  
  $form['#fid'] = $fid;
  
  return confirm_form(
    $form, t('Are you sure you want to delete %name?', array('%name' => $filters[$fid]['name'])),
    'admin/settings/customfilter',
    NULL,
    t('Delete')
  );
}

/**
 * Submission callback for the filter deleting form.
 */ 
function customfilter_filter_delete_submit($form, &$form_state) {
  $fid = $form['#fid'];
  
  db_query('DELETE FROM {customfilter_rules} WHERE fid = %d', $fid);
  db_query('DELETE FROM {customfilter_filters} WHERE fid = %d', $fid);
  customfilter_cache_clear($form_state['values']['fid']);
  
  $form_state['redirect'] = 'admin/settings/customfilter';
}

/**
 * Return the filter editing form.
 */
function customfilter_filter_edit($form_state, $op, $fid = 0) {
  if ($op == 'edit') {
    $filters = _customfilter_get_filters();
    $item = $filters[$fid];
    
    _customfilter_breadcrumb();
  }
  elseif ($op == 'add') {
    $fid = (integer) db_result(db_query("SELECT MAX(fid) FROM {customfilter_filters}")) + 1;
    $item = array(
      'fid' => $fid,
      'name' => t('Filter #!fid', array('!fid' => $fid)),
      'cache' => 0,
      'description' => '',
      'shorttips' => '',
      'longtips' => '',
    );
  }
   
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $item['fid'],
  );
  
  $form['operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );
  
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('The name of the filter.'),
    '#default_value' => $item['name'],
    '#required' => TRUE,
  );
  
  $form['cache'] = array(
    '#type' => 'checkbox',
    '#title' => t('Cache'),
    '#description' => t('If checked, the content will be cached.'),
    '#default_value' => $item['cache'],
  );
  
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('A description of the purpose of this filter.'),
    '#default_value' => $item['description'],
  );
  
  $form['shorttips'] = array(
    '#type' => 'textarea',
    '#title' => t('Tips (short)'),
    '#default_value' => $item['shorttips'],
    '#rows' => 5,
  );
  
  $form['longtips'] = array(
    '#type' => 'textarea',
    '#title' => t('Tips (full)'),
    '#default_value' => $item['longtips'],
    '#rows' => 20,
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  
  return $form;
}
 
/**
 * Submission callback for the filter editing form.
 */
function customfilter_filter_edit_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'edit':
      drupal_write_record('customfilter_filters', $form_state['values'], 'fid');
      break;
    
    case 'add':
      drupal_write_record('customfilter_filters', $form_state['values']);
      break;
  }
  
  customfilter_cache_clear($form_state['values']['fid']);
  
  $form_state['redirect'] = 'admin/settings/customfilter';
}
 
/** 
 * Filters list page.
 */
function customfilter_filters_list() {
  $header = array(
    t('Name'),
    t('Description'),
    array('data' => t('Operations'), 'colspan' => '3'),
  );
   
  $rows = array();
  
  foreach (_customfilter_get_filters() as $id => $filter) {
    $rows[] = array(
      l(t($filter['name']), "admin/settings/customfilter/{$id}"),
      '<em>'. t($filter['description']) .'</em>',
      l(t('delete'), "admin/settings/customfilter/{$id}/delete"),
      l(t('edit'), "admin/settings/customfilter/{$id}/edit"),
      l(t('test'), "admin/settings/customfilter/{$id}/test"),
    );
  }
  
  $output = theme(
    'table',
    $header,
    $rows ? $rows : array(array(array('data' => t('No filters have been defined.'), 'colspan' => 6)))
  );
  
  return $output;
}

/**
 * Return the import form.
 */
function customfilter_import_form() {
  $form = array();
  
  $form['import'] = array(
    '#type' => 'textarea',
    '#title' => t('Import data'),
    '#description' => t('Paste the text you obtained from the export form.'),
    '#cols' => 60,
    '#default_value' => '',
    '#rows' => 40,
    '#required' => TRUE,
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );
  
  return $form;
}
 
function customfilter_import_form_submit($form, &$form_state) {
  @eval($form_state['values']['import']);
  
  if (!isset($filter) || !isset($rules) || !is_array($filter) || !is_array($rules)) {
    form_set_error('import', t('The import data is not valid import text.'));
    return;
  }
  
  drupal_write_record('customfilter_filters', $filter);
  
  foreach ($rules as $rule) {
    drupal_write_record('customfilter_rules', $rule);
  }
  
  customfilter_cache_clear();
}

/**
 * Autocomplete callback.
 */
function customfilter_node_autocomplete($string = '') {
  $matches = array();
  
  if ($string) {
    $result = db_query_range("SELECT title FROM {node} WHERE title LIKE '%s%%'", $string, 0, 10);
    
    while ($node = db_fetch_object($result)) {
      $matches[$node->title] = check_plain($node->title);
    }
  }
  
  drupal_json($matches);
}

/**
 * Return the list of replacement rules form.
 */
function customfilter_rules_list(&$form_state, $fid, $rid = 0) {
  _customfilter_breadcrumb();
  
  $form['rules'] = _customfilter_rules_tree_form(_customfilter_get_rules($fid, $rid), 0);
  
  if (element_children($form)) {
    $form['fid'] = array(
      '#type' => 'hidden',
      '#value' => $fid,
    );
    
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  }
  else {
    $form['empty_rule'] = array(
      '#value' => t('There are no replacement rules yet.')
    );
  }
  
  return $form;
}

/**
 * Submission callback for the list of replacement rules form.
 */
function customfilter_rules_list_submit($form, &$form_state) {
  foreach ($form_state['values']['rules'] as $rule) {
    drupal_write_record('customfilter_rules', $rule, 'rid');
  }
  
  customfilter_cache_clear($form_state['values']['fid']);
  
  $form_state['redirect'] = "admin/settings/customfilter/{$form_state['values']['fid']}";
}

/**
 * Return the replacement rule delete form.
 */
function customfilter_rule_delete($form_state, $rid) {
  $filters = _customfilter_get_filters();
  $rule = db_fetch_object(db_query("SELECT * FROM {customfilter_rules} WHERE rid = %d", $rid));
  $rules_count = db_result(db_query("SELECT COUNT(*) FROM {customfilter_rules} WHERE prid = %d", $rid));
  
  if (isset($filters[$fid])) {
    _customfilter_breadcrumb(
      array(
        l($filters[$rule->fid]['name'], "admin/settings/customfilter/{$rule->fid}"),
      )
    );
  }
  
  $form['#rid'] = $rid;
  $form['#fid'] = $rule->fid;
  
  if ($rules_count) {
    $msg_text .= " <p>". t('This rule has subrules. If you delete this rule, they will be deleted too.') ."</p>";
  }

  $msg_text .= "<p>". t('This action cannot be undone.') ."</p>";
    
  return confirm_form(
    $form, t('Are you sure you want to delete the replacement rule %name?', array('%name' => $rule->name)),
    "admin/settings/customfilter/{$rule->fid}",
    $msg_text,
    t('Delete')
  );
}

/**
 * Submission callback for the replacement rule delete form.
 */
function customfilter_rule_delete_submit($form, &$form_state) {
  _customfilter_delete_rule($form['#rid']);
  customfilter_cache_clear($form_state['values']['fid']);
  
  $form_state['redirect'] = "admin/settings/customfilter/{$form['#fid']}";
}

/**
 * Return the replacement rule edit form.
 */
function customfilter_rule_edit($form_state, $op, $fid, $rid = 0) {
  $filters = _customfilter_get_filters();
  
  if (isset($filters[$fid])) {
    _customfilter_breadcrumb(
      array(
        l($filters[$fid]['name'], "admin/settings/customfilter/{$fid}"),
      )
    );
  }
  
  if ($op == 'edit') {
    $item = db_fetch_array(db_query("SELECT * FROM {customfilter_rules} WHERE rid = %d", $rid));
  }
  elseif ($op == 'add') {
    $item = array(
      'rid' => (integer) db_result(db_query("SELECT MAX(rid) FROM {customfilter_rules}")) + 1,
      'prid' => $rid,
      'fid' => $fid,
      'name' => '$1',
      'description' => '',
      'enabled' => 1,
      'matches' => 1,
      'pattern' => '/regex/i',
      'replacement' => 'Regular Expressions',
      'code' => 0,
      'weight' => 0,
    );
  }
  
  $matchopt = drupal_map_assoc(range(0, 99));
  
  $form['rid'] = array(
    '#type' => 'value',
    '#value' => $item['rid'],
  );
  
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $fid,
  );
  
  $form['prid'] = array(
    '#type' => 'value',
    '#value' => $item['prid'],
  );
  
  $form['operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );
  
  if ($item['prid'] != 0) {
    $form['matches'] = array(
      '#type' => 'select',
      '#title' => t('# Match'),
      '#description' => t('n-th matched substring in parent rule. This replacement rule will replace only for that substring.'),
      '#options' => $matchopt,
      '#default_value' => $item['matches'],
    );
  }
  
  $form['name'] = array(
    '#type' => 'textfield',
    '#description' => t('The name of this replacement rule.'),
    '#title' => t('Name'),
    '#default_value' => $item['name'],
    '#required' => TRUE,
  );
  
  $form['pattern'] = array(
    '#type' => 'textarea',
    '#title' => t('Pattern'),
    '#description' => t('The regular expression to use. Look at <a href="http://www.php.net/manual/en/regexp.reference.php">Regular Expression Details</a> for more help.'),
    '#default_value' => $item['pattern'],
    '#rows' => 3,
  );
  
  $form['code'] = array(
    '#type' => 'checkbox',
    '#title' => t('PHP Code'),
    '#description' => t('If selected, the replacement text is PHP code.'),
    '#default_value' => $item['code'],
  );
  
  $form['replacement']['replacement'] = array(
    '#type' => 'textarea',
    '#title' => t('Replacement text'),
    '#description' => t('The replacement text that will replace the matched string. Use $n (i.e. $1, $25) or ${n} (i.e. ${1}, ${25}), with n range from 0 to 99, to get the n-th original strings matched ($0 represents the entire matched string). If you select <strong>PHP Code</strong>, you can enter PHP code that will be executed during the elaboration of the rules. n-th matched string is provided in <code>$matches[n]</code>, and there is a global variable <code>$customfilter</code> (which contains a copy of the node object in <code>$customfilter->node</code>) you can use to store values that will be kept during the execution of different rules of the same filter. PHP code must set a value for <code>$result</code>, and must not be included between <code><?php</code> and <code>?></code>.'),
    '#default_value' => $item['replacement'],
    '#rows' => 16,
  );
  
  if (module_exists('token')) {
    $form['replacement']['token'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#description' => t('Prefer raw-text replacements for text to avoid problems with HTML entities.'),
      '#collapsed' => TRUE,
      '#collapsible' => TRUE,
    );
    
    $form['replacement']['token']['help'] = array(
      '#value' => theme('token_help', 'node'),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    );
  }
  
  $form['weight'] = array(
    '#title' => t('Weight'),
    '#default_value' => $item['weight'],
  );
  
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('The description of this rule.'),
    '#default_value' => $item['description'],
  );
  
  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t('If selected, the rule is used.'),
    '#default_value' => $item['enabled'],
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  
  return $form;
}
 
/**
 * Submission callback for the replacement rule edit form.
 */
function customfilter_rule_edit_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'edit':
      drupal_write_record('customfilter_rules', $form_state['values'], 'rid');
      break;
    
    case 'add':
      drupal_write_record('customfilter_rules', $form_state['values']);
      break;
  }
  
  customfilter_cache_clear($form_state['values']['fid']);
  
  $form_state['redirect'] = "admin/settings/customfilter/{$form_state['values']['fid']}";
}

function customfilter_test_form($form_state, $fid) {
  $filters = _customfilter_get_filters();
  $form = array();
  $options = array();
  
  if (isset($filters[$fid])) {
    _customfilter_breadcrumb(
      array(
        l($filters[$fid]['name'], "admin/settings/customfilter/{$fid}"),
      )
    );
    
    if (isset($form_state['storage']['output'])) {
      $output = $form_state['storage']['output'];
      
      $form['result'] = array(
        '#type' => 'fieldset',
        '#title' => t('Result'),
        '#collapsible' => TRUE,
      );
      
      $form['result']['result-output'] = array(
        '#value' => check_plain($output),
        '#prefix' => '<div>',
        '#suffix' => '</div>'
      );
      
      if (function_exists('_filter_htmlcorrector')) {
        $form['result']['preview'] = array(
          '#type' => 'fieldset',
          '#title' => t('Preview'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );
        
        if (($preview = _filter_htmlcorrector($output)) != $output) {
          $form['result']['preview']['message'] = array(
            '#value' => t('The result returned from the filter contains HTML tags left open.'),
            '#prefix' => '<div class="messages warning">',
            '#suffix' => '</div>'
          );
        }
        
        $form['result']['preview']['preview-output'] = array(
          '#value' => $preview,
          '#prefix' => '<div>',
          '#suffix' => '</div>'
        );
      }
    }
    
    foreach ($filters as $id => $filter) {
      $options[$id] = t($filter['name']);
    }
    
    if (!empty($options)) {
      $form['filter'] = array(
        '#type' => 'select',
        '#title' => t('Filter to test'),
        '#options' => $options,
        '#default_value' => $fid,
      );
      
      $form['trial'] = array(
        '#type' => 'fieldset',
        '#title' => t('Trial text'),
      );
      
      if (db_result(db_query("SELECT COUNT(*) FROM {node}"))) {
        $form['trial']['#description'] = t('Enter the title of the node, or the text to use to test the filter.');
        
        $form['trial']['node'] = array(
          '#type' => 'textfield',
          '#title' => t('Node title'),
          '#default_value' => '',
          '#autocomplete_path' => 'customfilter/node/autocomplete',
        );
      }
      else {
        $form['trial']['#description'] = t('Enter the text to use to test the filter.');
      }
      
      $text = isset($form_state['storage']['input']) ? $form_state['storage']['input'] : '';
      $rows = count(preg_split("/(\r\n?|\n)/", $text));
      
      $form['trial']['text'] = array(
        '#type' => 'textarea',
        '#default_value' => $text,
        "#rows" => max(15, $rows),
      );
      
      $form['op'] = array(
        '#type' => 'submit',
        '#value' => t('Test')
      );
    }
    else {
      drupal_set_message(t('There are no filters defined.'));
    }
  }
  elseif (isset($fid)) {
    drupal_set_message(t('An error occurred while loading the filter #!fid from the database', array('!fid' => $fid)));
    watchdog('customfilter', 'An error occurred while loading the filter #!fid from the database', array('!fid' => $fid), WATCHDOG_ERROR);
  }
  
  return $form;
}

function customfilter_test_form_submit($form, &$form_state) {
  $output = '';
  $nid = 0;
  $text = '';
  
  if (!empty($form_state['values']['node'])) {
    if (($nid = db_result(db_query("SELECT nid FROM {node} WHERE LOWER(title) = LOWER('%s')", $form_state['values']['node']))) && ($node = node_load($nid))) {
      $form_state['storage']['input'] = $text = $node->body;
    }
  }
  elseif (isset($form_state['values']['text'])) {
    $text = $form_state['values']['text'];
  }
  
  if ($text) {
    $output = customfilter_filter('process', $form_state['values']['filter'], -1, $text);
    $form_state['storage']['output'] = $output;
  }
}

/*****************************************************************************
 * Theme functions.
 ****************************************************************************/

/**
 * Theme for draggable tree of replacement rules
 */
function theme_customfilter_rules_list($form) {
  $header = array(
    t('Name'),
    t('Match'),
    t('Enabled'),
  );
  
  $header[] = array('data' => t('Operations'), 'colspan' => 3);
  
  $rows = array();
  foreach (element_children($form['rules']) as $rid) {
    $element =& $form['rules'][$rid];
    
    $element['prid']['#attributes']['class'] = 'rule-prid';
    $element['prid']['#type'] = 'hidden';
    $element['rid']['#attributes']['class'] = 'rule-rid';
    $element['weight']['#attributes']['class'] = 'rule-weight';
    $element['weight']['#type'] = 'hidden';
    
    $row['data'] = array(
      theme('indentation', $element['level']['#value']) . drupal_render($element['name']),
      drupal_render($element['matches']),
      drupal_render($element['enabled']),
      drupal_render($element['rid']) . drupal_render($element['prid']) . drupal_render($element['weight']),
    );
    
    if ($element['operations'])
      $row['data'] = array_merge(
        $row['data'],
        array(
          drupal_render($element['operations']['add']),
          drupal_render($element['operations']['edit']),
          drupal_render($element['operations']['delete']),
        )
      );
    
    $row['class'] = 'draggable';
    
    $rows[] = $row;
  }
  
  $output = theme('table', $header, $rows, array('id' => 'customfilter-rules-tree'));
  $output .= drupal_render($form);
  
  drupal_add_tabledrag('customfilter-rules-tree', 'match', 'parent', 'rule-prid', 'rule-prid', 'rule-rid', TRUE);
  drupal_add_tabledrag('customfilter-rules-tree', 'order', 'sibling', 'rule-weight', NULL, NULL, TRUE);
  
  return $output;
}

/*****************************************************************************
 * Private functions.
 ****************************************************************************/

/**
 * Map the imported filter ID to a new one.
 */
function _customfilter_map_filter_id($id) {
  static $fids = array();
  
  if (!isset($fids[$id])) {
    $fids[$id] = (integer) db_result(db_query("SELECT MAX(fid) FROM {customfilter_filters}")) + 1;
  }
  
  return $fids[$id];
}

/**
 * Map the imported rule ID to a new one.
 */
function _customfilter_map_rule_id($id) {
  static $rids = array();
  
  if (!$id) {
    return 0;
  }
  
  if (!isset($rids[$id])) {
    $rids[$id] = (integer) db_result(db_query("SELECT MAX(rid) FROM {customfilter_rules}")) + 1;
  }
  
  return $rids[$id];
}

/**
 * Helper function for customfilter_export_form.
 * Return the list made of a rule, and all the sub-rules.
 */
function _customfilter_rules_tree(&$rules, $fid, $prid = 0) {
  $result = db_query("SELECT * FROM {customfilter_rules} WHERE fid = %d and prid = %d ORDER BY weight", $fid, $prid);
  
  while ($rule = db_fetch_array($result)) {
    $rules[] = $rule;
    _customfilter_rules_tree($rules, $fid, $rule['rid']);
  }
}

/**
 * Return the rules tree form.
 */
function _customfilter_rules_tree_form($rules, $level = 0) {
  $base = 'admin/settings/customfilter';
  $form = array('#tree' => TRUE);
  
  foreach ($rules as $rule) {
    $fid = $rule['fid'];
    $rid = $rule['rid'];
    
    $form[$rid] = array(
      'name' => array(
        '#value' => l($rule['name'], "$base/$fid/$rid/edit"),
      ),
      'matches' => array(
        '#value' => ($rule['matches']? $rule['matches']: 'all'),
      ),
      'enabled' => array(
        '#type' => 'checkbox',
        '#default_value' => $rule['enabled'],
      ),
      'weight' => array(
        '#type' => 'weight',
        '#default_value' => $rule['weight'],
      ),
      'rid' => array(
        '#type' => 'hidden',
        '#value' => $rid,
      ),
      'prid' => array(
        '#type' => 'textfield',
        '#default_value' => $rule['prid'],
        '#size' => 4,
      ),
      'level' => array(
        '#type' => 'value',
        '#value' => $level,
      ),
    );
    
    $form[$rid]['operations'] = array(
      'add' => array(
        '#value' => l(t('add'), "$base/$fid/$rid/add"),
      ),
      'edit' => array(
        '#value' => l(t('edit'), "$base/$fid/$rid/edit"),
      ),
      'delete' => array(
        '#value' => l(t('delete'), "$base/$fid/$rid/delete"),
      ),
    );
    
    if ($rule['sub']) {
      $form += _customfilter_rules_tree_form($rule['sub'], $level + 1);
    }
  }
  
  return $form;
}

/**
 * @} End of "addtogroup customfilter".
 */